@@ -2,7 +2,7 @@
 
 # bounced.pl - This script runs as a daemon ; it does the incoming 
 # non-delivery reports analysis and storage
-# RCS Identication ; $Revision: 5816 $ ; $Date: 2009-05-27 11:14:41 +0200 (mer 27 mai 2009) $ 
+# RCS Identication ; $Revision: 6184 $ ; $Date: 2009-09-04 14:57:30 +0200 (ven 04 sep 2009) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -296,7 +296,7 @@ while (!$end) {
 		
 		if ($action =~ /do_it/i) {
 		    if ($list->is_user($who)) {
-			my $u = $list->delete_user($who);
+			my $u = $list->delete_user('users' => [$who], 'exclude' =>' 1');
 			do_log ('notice',"$who has been removed from $listname because welcome message bounced");
 			&Log::db_log({'robot' => $list->{'domain'},'list' => $list->{'name'},'action' => 'del',
 				      'target_email' => $who,'status' => 'error','error_type' => 'welcome_bounced',
@@ -409,7 +409,7 @@ while (!$end) {
 			$action = $result->{'action'} if (ref($result) eq 'HASH');		    
 			if ($action =~ /do_it/i) {
 			    if ($list->is_user($original_rcpt)) {
-				my $u = $list->delete_user($original_rcpt);
+				my $u = $list->delete_user('users' => [$original_rcpt], 'exclude' =>' 1');
 				
 				do_log ('notice',"$original_rcpt has been removed from %s because abuse feedback report",$list->name);	
 				unless ($list->send_notify_to_owner('automatic_del',{'who' => $original_rcpt, 'by' => 'listmaster'})) {
