@@ -1,5 +1,5 @@
 # Upgrade.pm - This module gathers all subroutines used to upgrade Sympa data structures
-#<!-- RCS Identication ; $Revision: 6132 $ --> 
+#<!-- RCS Identication ; $Revision: 6546 $ --> 
 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
@@ -662,6 +662,47 @@ sub upgrade {
 
    }
 
+   if (&tools::lower_version($previous_version, '6.1b.5')) {
+		## Encoding of shared documents was not consistent with recent versions of MIME::Encode
+		## MIME::EncWords::encode_mimewords() used to encode characters -!*+/ 
+		## Now these characters are preserved, according to RFC 2047 section 5 
+		## We change encoding of shared documents according to new algorithm
+		&do_log('notice','Fixing Q-encoding of web document filenames...');
+		my $all_lists = &List::get_lists('*');
+		foreach my $list ( @$all_lists ) {
+			if (-d $list->{'dir'}.'/shared') {
+				&do_log('notice','  Processing list %s...', $list->get_list_address());
+
+				my @all_files;
+				&tools::list_dir($list->{'dir'}, \@all_files, 'utf-8');
+				
+				my $count;
+				foreach my $f_struct (reverse @all_files) {
+					my $new_filename = $f_struct->{'filename'};
+					
+					## Decode and re-encode filename
+					$new_filename = &tools::qencode_filename(&tools::qdecode_filename($new_filename));
+					
+					if ($new_filename ne $f_struct->{'filename'}) {
+						## Rename file
+						my $orig_f = $f_struct->{'directory'}.'/'.$f_struct->{'filename'};
+						my $new_f = $f_struct->{'directory'}.'/'.$new_filename;
+						&do_log('notice', "Renaming %s to %s", $orig_f, $new_f);
+						unless (rename $orig_f, $new_f) {
+							&do_log('err', "Failed to rename %s to %s : %s", $orig_f, $new_f, $!);
+							next;
+						}
+						$count++;
+					}
+				}
+				if ($count) {
+				&do_log('notice', 'List %s : %d filenames has been changed', $list->{'name'}, $count);
+				}
+			}
+		}
+		
+   }	
+
     return 1;
 }
 
@@ -675,6 +716,7 @@ sub probe_db {
 						  'password_user' => 'varchar(40)',
 						  'last_login_date_user' => 'int(11)',
 						  'last_login_host_user' => 'varchar(60)',
+						  'wrong_login_count_user' => 'int(11)',
 						  'cookie_delay_user' => 'int(11)',
 						  'lang_user' => 'varchar(10)',
 						  'attributes_user' => 'text',
@@ -694,7 +736,10 @@ sub probe_db {
 							'include_sources_subscriber' => 'varchar(50)',
 							'bounce_score_subscriber' => 'smallint(6)',
 							'bounce_address_subscriber' => 'varchar(100)',
-							'custom_attribute_subscriber' => 'text'},
+							'custom_attribute_subscriber' => 'text',
+							'suspend_subscriber' => "int(1)",
+							'suspend_start_date_subscriber' => 'int(11)',
+							'suspend_end_date_subscriber' => 'int(11)'},
 				 'admin_table' => {'list_admin' => 'varchar(50)',
 						   'user_admin' => 'varchar(100)',
 						   'robot_admin' => 'varchar(80)',
@@ -709,6 +754,9 @@ sub probe_db {
 						   'include_sources_admin' => 'varchar(50)',
 						   'info_admin' =>  'varchar(150)',
 						   'profile_admin' => "enum('privileged','normal')"},
+				 'exclusion_table' => {'list_exclusion' => 'varchar(50)',
+						       'user_exclusion' => 'varchar(100)',
+						       'date_exclusion' => 'int(11)'},
 				 'netidmap_table' => {'netid_netidmap' => 'varchar(100)',
 						      'serviceid_netidmap' => 'varchar(100)',
 						      'email_netidmap' => 'varchar(100)',
@@ -742,20 +790,29 @@ sub probe_db {
 							     'remote_addr_one_time_ticket' => 'varchar(60)',
 							     'status_one_time_ticket' => 'varchar(60)'},
 				 'bulkmailer_table' => {'messagekey_bulkmailer' => 'varchar(80)',
+							'messageid_bulkmailer' => 'varchar(100)',
 							'packetid_bulkmailer' => 'varchar(33)',
 							'receipients_bulkmailer' => 'text',
 							'returnpath_bulkmailer' => 'varchar(100)',
 							'robot_bulkmailer' => 'varchar(80)',
 							'listname_bulkmailer' => 'varchar(50)',
 							'verp_bulkmailer' => 'int(1)',
+							'merge_bulkmailer' => 'int(1)',
 							'priority_message_bulkmailer' => 'smallint(10)',
 							'priority_packet_bulkmailer' => 'smallint(10)',
 							'reception_date_bulkmailer' => 'int(11)',
 							'delivery_date_bulkmailer' => 'int(11)',
 							'lock_bulkmailer' => 'varchar(30)'},
 				 'bulkspool_table' => {'messagekey_bulkspool' => 'varchar(33)',
+						       'messageid_bulkspool' => 'varchar(100)',
 						       'message_bulkspool' => 'longtext',
-						       'lock_bulkspool' => 'int(1)'},
+						       'lock_bulkspool' => 'int(1)',
+						       'dkim_privatekey_bulkspool' => 'varchar(1000)',
+						       'dkim_selector_bulkspool' => 'varchar(50)',
+						       'dkim_d_bulkspool' => 'varchar(50)',
+						       'dkim_i_bulkspool' => 'varchar(100)',
+						       'dkim_header_list_bulkspool' => 'varchar(500)',
+						   },
 				 'conf_table' => {'robot_conf' => 'varchar(80)',
 						  'label_conf' => 'varchar(80)',
 						  'value_conf' => 'varchar(300)'}
@@ -765,6 +822,7 @@ sub probe_db {
 						   'password_user' => 'text',
 						   'last_login_date_user' => 'integer',
 						   'last_login_host_user' => 'text',
+						   'wrong_login_count_user' => 'integer',
 						   'cookie_delay_user' => 'integer',
 						   'lang_user' => 'text',
 						   'attributes_user' => 'text',
@@ -784,7 +842,10 @@ sub probe_db {
 							 'include_sources_subscriber' => 'text',
 							 'bounce_score_subscriber' => 'integer',
 							 'bounce_address_subscriber' => 'text',
-							 'custom_attribute_subscriber' => 'text'},
+							 'custom_attribute_subscriber' => 'text',
+							 'suspend_subscriber' => 'numeric',
+							 'suspend_start_date_subscriber' => 'integer',
+							 'suspend_end_date_subscriber' => 'integer'},
 				  'admin_table' => {'list_admin' => 'text',
 						    'user_admin' => 'text',
 						    'robot_admin' => 'text',
@@ -799,6 +860,9 @@ sub probe_db {
 						    'include_sources_admin' => 'text',
 						    'info_admin' =>  'text',
 						    'profile_admin' => 'text'},
+				  'exclusion_table' => {'list_exclusion' => 'text',
+							'user_exclusion' => 'text',
+							'date_exclusion' => 'integer'},
 				  'netidmap_table' => {'netid_netidmap' => 'text',
 						       'serviceid_netidmap' => 'text',
 						       'email_netidmap' => 'text',
@@ -833,26 +897,32 @@ sub probe_db {
 						       'status_one_time_ticket' => 'text',				  
 							 },				 
 				  'bulkmailer_table' => {'messagekey_bulkmailer' => 'text',
+							 'messageid_bulkmailer' => 'text',
 							 'packetid_bulkmailer' => 'text',
 							 'receipients_bulkmailer' => 'text',
 							 'returnpath_bulkmailer' => 'text',
 							 'robot_bulkmailer' => 'text',
 							 'listname_bulkmailer' => 'text',
 							 'verp_bulkmailer' => 'integer',
+							 'merge_bulkmailer' => 'integer',
 							 'priority_message_bulkmailer' => 'integer',
 							 'priority_packet_bulkmailer' => 'integer',
 							 'reception_date_bulkmailer' => 'integer',
 							 'delivery_date_bulkmailer' => 'integer',
 							 'lock_bulkmailer' => 'text'},
 				  'bulkspool_table' => {'messagekey_bulkspool' => 'text',
+							'messageid_bulkspool' => 'text',
 							'message_bulkspool' => 'text',
-							'lock_bulkspool' => 'integer'},
+							'lock_bulkspool' => 'integer',
+							'dkim_privatekey_bulkspool' => 'text',
+							'dkim_selector_bulkspool' => 'text',
+							'dkim_d_bulkspool' => 'text',
+							'dkim_i_bulkspool' => 'text',
+							'dkim_header_list_bulkspool' => 'text'},
 				  'conf_table' => {'robot_conf' => 'text',
 						   'label_conf' => 'text',
-						   'value_conf' => 'text'}
-			      },
-		     );
-    
+						   'value_conf' => 'text'}});
+
     my %not_null = ('email_user' => 1,
 		    'list_subscriber' => 1,
 		    'robot_subscriber' => 1,
@@ -863,6 +933,8 @@ sub probe_db {
 		    'user_admin' => 1,
 		    'role_admin' => 1,
 		    'date_admin' => 1,
+		    'list_exclusion' => 1,
+		    'user_exclusion' => 1,
 		    'netid_netidmap' => 1,
 		    'serviceid_netidmap' => 1,
 		    'robot_netidmap' => 1,
@@ -882,6 +954,7 @@ sub probe_db {
     my %primary = ('user_table' => ['email_user'],
 		   'subscriber_table' => ['robot_subscriber','list_subscriber','user_subscriber'],
 		   'admin_table' => ['robot_admin','list_admin','role_admin','user_admin'],
+		   'exclusion_table' => ['list_exclusion','user_exclusion'],
 		   'netidmap_table' => ['netid_netidmap','serviceid_netidmap','robot_netidmap'],
 		   'logs_table' => ['id_logs'],
 		   'session_table' => ['id_session'],
@@ -998,7 +1071,8 @@ sub probe_db {
  	}
 	
  	foreach my $t (@tables) {
- 	    $t =~ s/^\"(.+)\"$/$1/;
+	    $t =~ s/^"main"\.//; # needed for SQLite 3
+	    $t =~ s/^.*\"([^"]+)\"$/$1/;
  	}
 	
 	foreach my $t (@tables) {
@@ -1287,7 +1361,7 @@ sub probe_db {
 		    }
 		}	 
 	    }   
-	    elsif ($Conf::Conf{'db_type'} eq 'sqlite') {
+	    elsif ($Conf::Conf{'db_type'} eq 'SQLite') {
 		## Create required INDEX and PRIMARY KEY
 		my $should_update;
 		foreach my $field (@{$primary{$t}}) {
@@ -1299,28 +1373,10 @@ sub probe_db {
 		
 		if ($should_update) {
 		    my $fields = join ',',@{$primary{$t}};
-		    
-		    ## drop previous primary key
-		    unless ($dbh->do("ALTER TABLE $t DROP PRIMARY KEY")) {
-			&do_log('err', 'Could not drop PRIMARY KEY, table\'%s\'.', $t);
-		    }
-		    push @report, sprintf('Table %s, PRIMARY KEY dropped', $t);
-		    &do_log('info', 'Table %s, PRIMARY KEY dropped', $t);
-		    
-		    ## Add primary key
-		    &do_log('debug', "ALTER TABLE $t ADD PRIMARY KEY ($fields)");
-		    unless ($dbh->do("ALTER TABLE $t ADD PRIMARY KEY ($fields)")) {
-			&do_log('err', 'Could not set field \'%s\' as PRIMARY KEY, table\'%s\'.', $fields, $t);
-			return undef;
-		    }
-		    push @report, sprintf('Table %s, PRIMARY KEY set on %s', $t, $fields);
-		    &do_log('info', 'Table %s, PRIMARY KEY set on %s', $t, $fields);
-		    
-		    
 		    ## drop previous index
 		    my $success;
 		    foreach my $field (@{$primary{$t}}) {
-			unless ($dbh->do("ALTER TABLE $t DROP INDEX $field")) {
+			unless ($dbh->do("DROP INDEX $field")) {
 			    next;
 			}
 			$success = 1; last;
@@ -1334,7 +1390,7 @@ sub probe_db {
 		    }
 		    
 		    ## Add INDEX
-		    unless ($dbh->do("ALTER TABLE $t ADD INDEX $t\_index ($fields)")) {
+		    unless ($dbh->do("CREATE INDEX IF NOT EXIST $t\_index ON $t ($fields)")) {
 			&do_log('err', 'Could not set INDEX on field \'%s\', table\'%s\'.', $fields, $t);
 			return undef;
 		    }
