@@ -338,7 +338,7 @@ sub check_cpan {
 					  usage => 'Sybase database driver, required if you connect to a Sybase database.',
 					 },
 			'DBD::mysql' => {
-					 required_version =>'2.0407',
+					 required_version =>'4.008',
 					 prerequisites => 'mysql-devel and myslq-server. mysql should be running for make test to succeed',
 					 package_name => 'Msql-Mysql-modules',
 					 mandatory => 1,
@@ -383,7 +383,7 @@ sub check_cpan {
 					       usage => 'used to compute plaindigest messages from HTML',
 					      },
 			'HTML::StripScripts::Parser' => {
-							 required_version =>'1.0',
+							 required_version =>'1.03',
 							 package_name => 'HTML-StripScripts-Parser',
 							 mandatory => 1,
 							 usage => 'required for XSS protection on the web interface',
@@ -426,13 +426,13 @@ sub check_cpan {
 					   usage => 'required to compute digest for password and emails',
 					  },
 			'MIME::Charset' => {
-					    required_version =>'0.04.1',
+					    required_version =>'1.006.2',
 					    package_name => 'MIME-Charset',
 					    mandatory => 1,
 					    usage => 'used to encode mail body using a different charset',
 					   },
 			'MIME::EncWords' => {
-					     required_version =>'0.040',
+					     required_version =>'1.010',
 					     package_name => 'MIME-EncWords',
 					     mandatory => 1,
 					     usage => 'required to decode/encode SMTP header fields without breaking character encoding', 
@@ -484,21 +484,36 @@ sub check_cpan {
 						mandatory => 1,	
 						usage => 'used while checking the RDBMS buffer size',
 					       },
+			'Text::LineFold' => {
+			    		     required_version =>'0.008',
+					     package_name => 'Unicode-LineBreak',
+					     mandatory => 1,
+					     usage => 'used to fold lines in HTML mail composer and system messages, prior to Text::Wrap',
+					    },
 			'Time::HiRes' => {
-					  required_version =>'1.9719',
+					  required_version =>'1.29',
 					  package_name => 'Time-HiRes',
 					  mandatory => 1,
 					  usage => 'used by sympa.pl --test_database_message_buffer to test database performances',
 					 },
+			'URI::Escape' => {
+					  required_version =>'1.35',
+					  package_name => 'URI-Escape',
+					  mandatory => 1,
+					  usage => 'Used to create URI containing non URI-canonical characters.',
+					 },
 			'XML::LibXML' => {
 					  prerequisites => 'libxml2-devel is needed to build the Perl code',
 					  package_name => 'XML-LibXML',
 					  mandatory => 1,
 					  usage => 'used to parse list configuration templates and instanciate list families',
 					 },
+			'Mail::DKIM' => { required_version =>'0.36',
+					  package_name => 'Mail-DKIM',
+					  usage => 'required in order to use DKIM features (both for signature verification and signature insertion)',
+				      } 
 		       );
 
-
     print "##########################################################################################
 # This process will help you install all Perl (CPAN) modules required by Sympa software.
 # Sympa requires from 50 to 65 additional Perl modules to run properly. 
@@ -511,6 +526,49 @@ Strike return key to continue...
 ";
     my $rep = <STDIN>;
 
+    my %opt_CPAN = (
+		'AuthCAS' => 'AuthCAS',
+        'DBD::Pg' => 'DBD-Pg',
+        'DBD::Oracle' => 'DBD-Oracle',
+        'DBD::Sybase' => 'DBD-Sybase',
+        'DBD::SQLite' => 'DBD-SQLite',
+        'Net::LDAP' =>   'perl-ldap',
+        'CGI::Fast' => 'CGI',
+        'Net::SMTP' => 'libnet',
+        'IO::Socket::SSL' => 'IO-Socket-SSL',
+        'Net::SSLeay' => 'NET-SSLeay',
+        'Bundle::LWP' => 'LWP',
+        'SOAP::Lite' => 'SOAP-Lite',
+        'File::NFSLock' => 'File-NFSLock',
+        'File::Copy::Recursive' => 'File-Copy-Recursive',
+        'Crypt::CipherSaber' => 'CipherSaber',
+	'mail::DKIM' =>'Mail-DKIM',
+    );
+
+    my %opt_features = (
+	'AuthCAS' => 'CAS Single Sign-On client libraries. Required if you configure Sympa to delegate web authentication to a CAS server.',
+        'DBI' => 'a generic Database Driver, required by Sympa to access Subscriber information and User preferences. An additional Database Driver is required for each database type you wish to connect to.',
+        'DBD::mysql' => 'Mysql database driver, required if you connect to a Mysql database.\nYou first need to install the Mysql server and have it started before installing the Perl DBD module.',
+        'DBD::Pg' => 'PostgreSQL database driver, required if you connect to a PostgreSQL database.',
+        'DBD::Oracle' => 'Oracle database driver, required if you connect to a Oracle database.',
+        'DBD::Sybase' => 'Sybase database driver, required if you connect to a Sybase database.',
+        'DBD::SQLite' => 'SQLite database driver, required if you connect to a SQLite database.',
+        'Net::LDAP' =>   'required to query LDAP directories. Sympa can do LDAP-based authentication ; it can also build mailing lists with LDAP-extracted members.',
+        'CGI::Fast' => 'WWSympa, Sympa\'s web interface can run as a FastCGI (ie: a persistent CGI). If you install this module, you will also need to install the associated mod_fastcgi for Apache.',
+        'Crypt::CipherSaber' => 'this module provides reversible encryption of user passwords in the database.usefull only when updating from old version with password reversible encryption.',
+        'Archive::Zip ' => 'this module provides zip/unzip for archive and shared document download/upload',
+        'FCGI' => 'WSympa, Sympa\'s web interface can run as a FastCGI (ie: a persistent CGI). If you install this module, you will also need to install the associated mod_fastcgi for Apache.',
+        'Net::SMTP' => 'this is required if you set \'list_check_smtp\' sympa.conf parameter, used to check existing aliases before mailing list creation.',
+        'IO::Socket::SSL' => 'required by CAS (single sign-on) and the \'include_remote_sympa_list\' feature that includes members of a list on a remote server, using X509 authentication',
+        'Net::SSLeay' => 'required by the \'include_remote_sympa_list\' feature that includes members of a list on a remote server, using X509 authentication',
+        'Bundle::LWP' => 'required by the \'include_remote_sympa_list\' feature that includes members of a list on a remote server, using X509 authentication',
+        'SOAP::Lite' => 'required if you want to run the Sympa SOAP server that provides ML services via a "web service"',
+        'File::NFSLock' => 'required to perform NFS lock ; see also lock_method sympa.conf parameter',
+	'mail::DKIM' => 'requiered in order to use Sympa DKIM features, both for signature verification  and signature insertion. it should be used event if you do not want your server to add signature !',
+    );
+
+    ### main:
+    print "******* Check perl for SYMPA ********\n";
     ### REQ perl version
     print "\nChecking for PERL version:\n-----------------------------\n";
     my $rpv = $cpan_modules{"perl"}{'required_version'};
